//
// "$Id$"
//
// Platform abstraction header file for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2018 by Bill Spitzak and others.
//
// This library is free software. Distribution and use rights are outlined in
// the file "COPYING" which should have been included with this file.  If this
// file is missing or damaged, see the license at:
//
//     http://www.fltk.org/COPYING.php
//
// Please report all bugs and problems on the following page:
//
//     http://www.fltk.org/str.php
//

// This file is present for compatibility with FLTK 1.4 and later.

// In FLTK 1.4 FL/platform.H replaces FL/x.H.  FLTK 1.4 code that
// includes FL/platform.H instead of FL/x.H can now be compiled with
// FLTK 1.3.5 and later versions.

#if !defined(Fl_X_H)
#include <FL/x.H>
#endif

//
// End of "$Id$".
//
