//
// "$Id$"
//
// RGB Image header file for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2010 by Bill Spitzak and others.
//
// This library is free software. Distribution and use rights are outlined in
// the file "COPYING" which should have been included with this file.  If this
// file is missing or damaged, see the license at:
//
//     http://www.fltk.org/COPYING.php
//
// Please report all bugs and problems on the following page:
//
//     http://www.fltk.org/str.php
//

#ifndef Fl_RGB_Image_H
# define Fl_RGB_Image_H
# include "Fl_Image.H"
#endif // !Fl_RGB_Image_H

//
// End of "$Id$".
//
